from design.plone.ctgeneric import _
from plone import api
from zope.i18n import translate


def get_design_meta_type(self):
    ttool = api.portal.get_tool("portal_types")
    tipologia_notizia = getattr(self.context, "tipologia_notizia", "")
    if self.context.portal_type == "News Item" and tipologia_notizia:
        return translate(
            tipologia_notizia,
            domain=_._domain,
            context=self.request,
        )
    return translate(ttool[self.context.portal_type].Title(), context=self.request)



from design.plone.ctgeneric.interfaces import IDesignPloneV2Settings
from plone import api

import json
import logging
import six


logger = logging.getLogger(__name__)


def get_settings_for_language(field):
    values = api.portal.get_registry_record(
        field, interface=IDesignPloneV2Settings, default=[]
    )
    if not values:
        return []
    if not isinstance(values, six.text_type):
        return values
    try:
        json_data = json.loads(values)
    except Exception as e:
        logger.exception(e)
        return values
    lang = api.portal.get_current_language()
    return json_data.get(lang, [])



# -*- coding: utf-8 -*-
"""Module where all interfaces, events and exceptions live."""
from design.plone.contenttypes.interfaces import IDesignPloneContenttypesLayer


class IDesignPloneCtgenericLayer(IDesignPloneContenttypesLayer):
    """Marker interface that defines a browser layer."""



# -*- coding: utf-8 -*-
from design.plone.contenttypes.tests.test_ct_persona import (
    TestPersonaSchema as BaseSchemaTest,
)
from design.plone.ctgeneric.testing import DESIGN_PLONE_CTGENERIC_API_FUNCTIONAL_TESTING
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.restapi.testing import RelativeSession
from transaction import commit
from z3c.relationfield import RelationValue
from zope.component import getUtility
from zope.intid.interfaces import IIntIds

import unittest


class TestPersonaSchema(BaseSchemaTest):
    layer = DESIGN_PLONE_CTGENERIC_API_FUNCTIONAL_TESTING

    def test_behaviors_enabled_for_persona(self):
        portal_types = api.portal.get_tool(name="portal_types")
        self.assertEqual(
            portal_types["Persona"].behaviors,
            (
                "plone.namefromtitle",
                "plone.allowdiscussion",
                "plone.excludefromnavigation",
                "plone.shortname",
                "plone.ownership",
                "plone.publication",
                "plone.relateditems",
                "plone.categorization",
                "plone.basic",
                "plone.locking",
                "design.plone.contenttypes.behavior.additional_help_infos",
                "plone.textindexer",
                "plone.translatable",
                "kitconcept.seo",
                "plone.versioning",
                "design.plone.contenttypes.behavior.persona_v2",
            ),
        )

    def test_persona_required_fields(self):
        resp = self.api_session.get("@types/Persona").json()
        self.assertEqual(
            sorted(resp["required"]),
            sorted(["title", "ruolo", "tipologia_persona"]),
        )

    def test_persona_fields_ruolo_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Persona").json()
        self.assertEqual(
            resp["fieldsets"][1]["fields"],
            [
                "ruolo",
                "organizzazione_riferimento",
                "data_conclusione_incarico",
                "competenze",
                "deleghe",
                "tipologia_persona",
                "data_insediamento",
                "biografia",
            ],
        )

    def test_persona_fields_contatti_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Persona").json()
        self.assertEqual(resp["fieldsets"][2]["fields"], ["telefono", "fax", "email"])

    def test_persona_fields_documenti_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Persona").json()
        self.assertEqual(
            resp["fieldsets"][3]["fields"], ["curriculum_vitae", "atto_nomina"]
        )


class TestPersonaEndpoint(unittest.TestCase):
    """"""

    layer = DESIGN_PLONE_CTGENERIC_API_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)

        self.persona = api.content.create(
            container=self.portal, type="Persona", title="John Doe"
        )
        intids = getUtility(IIntIds)

        self.persona_ref = RelationValue(intids.getId(self.persona))
        commit()

    def tearDown(self):
        self.api_session.close()

    def test_persona_strutture_correlate(self):
        uo = api.content.create(
            container=self.portal,
            type="UnitaOrganizzativa",
            title="UO 1",
            persone_struttura=[self.persona_ref],
        )
        commit()
        response = self.api_session.get(self.persona.absolute_url())
        res = response.json()

        self.assertIn("strutture_correlate", list(res.keys()))
        self.assertEqual(len(res["strutture_correlate"]), 1)
        self.assertEqual(res["strutture_correlate"][0]["title"], uo.title)

    def test_persona_responsabile_di(self):
        uo = api.content.create(
            container=self.portal,
            type="UnitaOrganizzativa",
            title="UO 1",
            responsabile=[self.persona_ref],
        )
        commit()
        response = self.api_session.get(self.persona.absolute_url())
        res = response.json()

        self.assertIn("responsabile_di", list(res.keys()))
        self.assertEqual(len(res["responsabile_di"]), 1)
        self.assertEqual(res["responsabile_di"][0]["title"], uo.title)

    def test_persona_assessore_di(self):
        uo = api.content.create(
            container=self.portal,
            type="UnitaOrganizzativa",
            title="UO 1",
            assessore_riferimento=[self.persona_ref],
        )
        commit()
        response = self.api_session.get(self.persona.absolute_url())
        res = response.json()

        self.assertIn("assessore_di", list(res.keys()))
        self.assertEqual(len(res["assessore_di"]), 1)
        self.assertEqual(res["assessore_di"][0]["title"], uo.title)

    def test_persona_substructure_created(self):
        self.api_session.post(
            self.portal_url,
            json={"@type": "Persona", "title": "John"},
        )

        commit()
        persona = self.portal["john"]
        self.assertEqual(
            [
                "foto-e-attivita-politica",
                "curriculum-vitae",
                "situazione-patrimoniale",
                "dichiarazione-dei-redditi",
                "spese-elettorali",
                "variazione-situazione-patrimoniale",
                "altre-cariche",
                "compensi",
                "importi-di-viaggio-e-o-servizi",
            ],
            persona.keys(),
        )



# -*- coding: utf-8 -*-
from design.plone.contenttypes.tests.test_ct_modulo import (
    TestModuloSchema as BaseSchemaTest,
)
from design.plone.ctgeneric.testing import DESIGN_PLONE_CTGENERIC_API_FUNCTIONAL_TESTING


class TestModuloSchema(BaseSchemaTest):
    layer = DESIGN_PLONE_CTGENERIC_API_FUNCTIONAL_TESTING


# class TestModuloSchema(unittest.TestCase):
#     layer = DESIGN_PLONE_CONTENTTYPES_API_FUNCTIONAL_TESTING

#     def setUp(self):
#         self.app = self.layer["app"]
#         self.portal = self.layer["portal"]
#         self.request = self.layer["request"]
#         self.portal_url = self.portal.absolute_url()
#         setRoles(self.portal, TEST_USER_ID, ["Manager"])

#         self.api_session = RelativeSession(self.portal_url)
#         self.api_session.headers.update({"Accept": "application/json"})
#         self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)

#     def tearDown(self):
#         self.api_session.close()

#     def test_behaviors_enabled_for_modulo(self):
#         portal_types = api.portal.get_tool(name="portal_types")
#         self.assertEqual(
#             portal_types["Modulo"].behaviors,
#             (
#                 "plone.namefromtitle",
#                 "plone.allowdiscussion",
#                 "plone.excludefromnavigation",
#                 "plone.shortname",
#                 "plone.dublincore",
#                 "plone.relateditems",
#                 "plone.locking",
#                 "design.plone.contenttypes.behavior.multi_file",
#                 "plone.translatable",
#             ),
#         )

#     def test_modulo_fieldsets(self):
#         """
#         Get the list from restapi
#         """
#         resp = self.api_session.get("@types/Modulo").json()
#         self.assertEqual(len(resp["fieldsets"]), 5)
#         self.assertEqual(
#             [x.get("id") for x in resp["fieldsets"]],
#             [
#                 "default",
#                 "settings",
#                 # "correlati",
#                 "categorization",
#                 "dates",
#                 "ownership",
#             ],
#         )

#     def test_modulo_required_fields(self):
#         resp = self.api_session.get("@types/Modulo").json()
#         self.assertEqual(
#             sorted(resp["required"]),
#             sorted(["title", "file_principale"]),
#         )

#     def test_modulo_fields_default_fieldset(self):
#         """
#         Get the list from restapi
#         """
#         resp = self.api_session.get("@types/Modulo").json()
#         self.assertEqual(
#             resp["fieldsets"][0]["fields"],
#             [
#                 "title",
#                 "description",
#                 "file_principale",
#                 "formato_alternativo_1",
#                 "formato_alternativo_2",
#             ],
#         )

#     def test_modulo_fields_settings_fieldset(self):
#         """
#         Get the list from restapi
#         """
#         resp = self.api_session.get("@types/Modulo").json()
#         self.assertEqual(
#             resp["fieldsets"][1]["fields"],
#             [
#                 "allow_discussion",
#                 "exclude_from_nav",
#                 "id",
#             ],
#         )

#     def test_modulo_fields_categorization_fieldset(self):
#         """
#         Get the list from restapi
#         """
#         resp = self.api_session.get("@types/Modulo").json()
#         self.assertEqual(
#             resp["fieldsets"][2]["fields"],
#             # ["subjects", "language"] BBB dovrebbe essere così
#             # ma nei test esce così perché non viene vista la patch di SchemaTweaks
#             ["subjects", "language", "relatedItems"],
#         )

#     def test_modulo_fields_dates_fieldset(self):
#         """
#         Get the list from restapi
#         """
#         resp = self.api_session.get("@types/Modulo").json()
#         self.assertEqual(resp["fieldsets"][3]["fields"], ["effective", "expires"])

#     def test_modulo_fields_ownership_fieldset(self):
#         """
#         Get the list from restapi
#         """
#         resp = self.api_session.get("@types/Modulo").json()
#         self.assertEqual(
#             resp["fieldsets"][4]["fields"], ["creators", "contributors", "rights"]
#         )



# -*- coding: utf-8 -*-
from design.plone.ctgeneric import _
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.interfaces import IDexterityContent
from plone.supermodel import model
from zope import schema
from zope.component import adapter
from zope.interface import implementer
from zope.interface import provider


@provider(IFormFieldProvider)
class ILuogoV2(model.Schema):
    riferimento_telefonico_struttura = schema.TextLine(
        title=_(
            "riferimento_telefonico_struttura",
            default="Telefono della struttura responsabile",
        ),
        description=_(
            "help_riferimento_telefonico_struttura",
            default="Indicare il riferimento telefonico per poter contattare"
            " i referenti della struttura responsabile.",
        ),
        required=False,
    )
    riferimento_fax_struttura = schema.TextLine(
        title=_(
            "riferimento_fax_struttura",
            default="Fax della struttura responsabile",
        ),
        description=_(
            "help_riferimento_fax_struttura",
            default="Indicare un numero di fax della struttura responsabile.",
        ),
        required=False,
    )
    riferimento_mail_struttura = schema.TextLine(
        title=_(
            "riferimento_mail_struttura",
            default="E-mail struttura responsabile",
        ),
        description=_(
            "help_riferimento_mail_struttura",
            default="Indicare un indirizzo mail per poter contattare"
            " i referenti della struttura responsabile.",
        ),
        required=False,
    )

    riferimento_pec_struttura = schema.TextLine(
        title=_(
            "riferimento_pec_struttura",
            default="Pec della struttura responsabile",
        ),
        description=_(
            "help_riferimento_pec_struttura",
            default="Indicare un indirizzo pec per poter contattare"
            " i referenti della struttura responsabile.",
        ),
        required=False,
    )

    model.fieldset(
        "contatti",
        label=_("contatti_label", default="Contatti"),
        fields=[
            "riferimento_telefonico_struttura",
            "riferimento_fax_struttura",
            "riferimento_mail_struttura",
            "riferimento_pec_struttura",
        ],
    )


@implementer(ILuogoV2)
@adapter(IDexterityContent)
class LuogoV2(object):
    """ """

    def __init__(self, context):
        self.context = context



# -*- coding: utf-8 -*-
from design.plone.contenttypes.tests.test_ct_servizio import (
    TestServizioSchema as BaseSchemaTest,
)
from design.plone.ctgeneric.testing import DESIGN_PLONE_CTGENERIC_API_FUNCTIONAL_TESTING
from plone import api


class TestServizioSchema(BaseSchemaTest):
    layer = DESIGN_PLONE_CTGENERIC_API_FUNCTIONAL_TESTING

    def test_behaviors_enabled_for_servizio(self):
        portal_types = api.portal.get_tool(name="portal_types")
        self.assertEqual(
            portal_types["Servizio"].behaviors,
            (
                "plone.namefromtitle",
                "plone.allowdiscussion",
                "plone.excludefromnavigation",
                "plone.shortname",
                "plone.ownership",
                "plone.publication",
                "plone.categorization",
                "plone.basic",
                "design.plone.contenttypes.behavior.descrizione_estesa_servizio",
                "plone.leadimage",
                "volto.preview_image",
                "plone.relateditems",
                "design.plone.contenttypes.behavior.additional_help_infos",
                "plone.textindexer",
                "plone.translatable",
                "kitconcept.seo",
                "plone.versioning",
                "plone.locking",
                "design.plone.contenttypes.behavior.argomenti",
                "design.plone.contenttypes.behavior.servizio_v2",
            ),
        )

    def test_servizio_fieldsets(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Servizio").json()
        self.assertEqual(len(resp["fieldsets"]), 18)
        self.assertEqual(
            [x.get("id") for x in resp["fieldsets"]],
            [
                "default",
                "cose",
                "a_chi_si_rivolge",
                "accedi_al_servizio",
                "cosa_serve",
                "costi_e_vincoli",
                "tempi_e_scadenze",
                "casi_particolari",
                "contatti",
                "documenti",
                "link_utili",
                "informazioni",
                "correlati",
                "categorization",
                "settings",
                "ownership",
                "dates",
                "seo",
            ],
        )

    def test_servizio_required_fields(self):
        resp = self.api_session.get("@types/Servizio").json()
        self.assertEqual(
            sorted(resp["required"]),
            sorted(["title", "cosa_serve", "ufficio_responsabile"]),
        )

    def test_servizio_fields_default_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Servizio").json()
        self.assertEqual(
            resp["fieldsets"][0]["fields"],
            [
                "title",
                "description",
                "sottotitolo",
                "stato_servizio",
                "motivo_stato_servizio",
                "image",
                "image_caption",
                "preview_image",
                "preview_caption",
                "tassonomia_argomenti",
            ],
        )

    def test_servizio_fields_a_chi_si_rivolge_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Servizio").json()
        self.assertEqual(
            resp["fieldsets"][2]["fields"],
            ["a_chi_si_rivolge", "chi_puo_presentare", "copertura_geografica"],
        )

    def test_servizio_fields_accedi_al_servizio_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Servizio").json()
        self.assertEqual(
            resp["fieldsets"][3]["fields"],
            [
                "come_si_fa",
                "cosa_si_ottiene",
                "procedure_collegate",
                "canale_digitale",
                "autenticazione",
                "dove_rivolgersi",
                "dove_rivolgersi_extra",
                "prenota_appuntamento",
            ],
        )

    def test_servizio_fields_tempi_e_scadenze_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Servizio").json()
        self.assertEqual(
            resp["fieldsets"][6]["fields"],
            ["tempi_e_scadenze"],
        )

    def test_servizio_fields_contatti_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Servizio").json()
        self.assertEqual(
            resp["fieldsets"][8]["fields"],
            ["ufficio_responsabile", "area"],
        )

    def test_servizio_fields_correlati_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Servizio").json()
        self.assertEqual(
            resp["fieldsets"][12]["fields"],
            ["servizi_collegati", "correlato_in_evidenza"],
        )



from design.plone.contenttypes.interfaces.documento import IDocumento
from design.plone.contenttypes.interfaces.persona import IPersona
from design.plone.contenttypes.interfaces.servizio import IServizio
from design.plone.contenttypes.interfaces.unita_organizzativa import IUnitaOrganizzativa
from plone.autoform.interfaces import IFormFieldProvider
from plone.autoform.interfaces import OMITTED_KEY
from plone.supermodel.interfaces import FIELDSETS_KEY
from plone.supermodel.interfaces import ISchemaPlugin
from plone.supermodel.model import Fieldset
from zope.component import adapter
from zope.interface import implementer
from zope.interface import Interface

import logging


logger = logging.getLogger(__name__)


@implementer(ISchemaPlugin)
@adapter(IFormFieldProvider)
class SchemaTweaks(object):
    """
    Fix fields for content-types to be like v2 of design.plone.contenttypes
    """

    order = 99999

    def __init__(self, schema):
        self.schema = schema

    def __call__(self):
        self.fix_documento_schema()
        self.fix_event_schema()
        self.fix_news_schema()
        self.fix_persona_schema()
        self.fix_servizio_schema()
        self.fix_unita_organizzativa_schema()
        self.fix_luogo_schema()

    def move_field(
        self,
        schema,
        fieldname,
        to_fieldset_name,
        label=None,
        description=None,
        order=None,
    ):
        """
        Copied from https://community.plone.org/t/moving-behavior-fields-to-different-fieldset/6219/7
        Moves a field named "fieldname" on a Zope "schema" to a new fieldset "to_field_name".

        - creates a new fieldset on demand (then label, description and order are passed to the new one).
        - if value of "to_fieldset_name" is "default", then the field sticks on the main form.
        """
        # find schema with field in inheritance tree
        schema_with_field = None
        for pschema in reversed(schema.__iro__):
            if pschema.direct(fieldname):
                schema_with_field = pschema
                break
        if schema_with_field is None:
            raise KeyError(f"field '{fieldname}' does not exist on {schema}.")

        # remove field from fieldset (if in any)
        fieldsets_direct = schema_with_field.queryDirectTaggedValue(FIELDSETS_KEY)
        if fieldsets_direct is not None:
            for fieldset in fieldsets_direct:
                if fieldname in fieldset.fields:
                    fieldset.fields.remove(fieldname)
                    break

        if to_fieldset_name == "default":
            # default means to fieldset, but on main form
            return

        if fieldsets_direct is None:
            # no tagged value set so far!
            fieldsets_direct = list()
            schema.setTaggedValue(FIELDSETS_KEY, fieldsets_direct)

        # try to find the fieldset, append and exit
        for fieldset in fieldsets_direct:
            if fieldset.__name__ == to_fieldset_name:
                fieldset.fields.append(fieldname)
                return

        # not found, need to create new fieldset
        new_fieldset = Fieldset(
            to_fieldset_name,
            fields=[fieldname],
        )
        if label is not None:
            new_fieldset.label = label
        if description is not None:
            new_fieldset.description = description
        if order is not None:
            new_fieldset.order = order
        fieldsets_direct.append(new_fieldset)
        # Done!

    def fix_documento_schema(self):
        """fix Documento fields"""
        IDocumento["formati_disponibili"].required = False
        IDocumento["ufficio_responsabile"].required = False
        IDocumento.setTaggedValue(
            OMITTED_KEY,
            [
                (Interface, "protocollo", "true"),
                (Interface, "data_protocollo", "true"),
                (Interface, "formati_disponibili", "true"),
                (Interface, "dataset", "true"),
            ],
        )
        if self.schema.getName() == "IArgomentiDocumento":
            self.schema["tassonomia_argomenti"].required = False

    def fix_event_schema(self):
        """fix Documento fields"""
        if self.schema.getName() == "IArgomentiEvento":
            self.schema["tassonomia_argomenti"].required = False
            self.move_field(
                schema=self.schema,
                fieldname="correlato_in_evidenza",
                to_fieldset_name="correlati",
            )
        if self.schema.getName() == "IEvento":
            self.schema["descrizione_estesa"].required = False
            self.schema["descrizione_destinatari"].required = False
            self.move_field(
                schema=self.schema,
                fieldname="patrocinato_da",
                to_fieldset_name="informazioni",
            )

    def fix_news_schema(self):
        """fix News Item fields"""
        if self.schema.getName() == "IArgomentiNews":
            self.schema["tassonomia_argomenti"].required = False
        if self.schema.getName() == "INewsAdditionalFields":
            self.schema["descrizione_estesa"].required = False
            self.schema["a_cura_di"].required = False
            self.move_field(
                schema=self.schema,
                fieldname="notizie_correlate",
                to_fieldset_name="default",
            )

    def fix_persona_schema(self):
        """fix Persona fields"""
        IPersona.setTaggedValue(OMITTED_KEY, [(Interface, "incarichi_persona", "true")])

    def fix_servizio_schema(self):
        """fix Persona fields"""
        if self.schema.getName() == "IArgomentiServizio":
            self.schema["tassonomia_argomenti"].required = False
            self.move_field(
                schema=self.schema,
                fieldname="correlato_in_evidenza",
                to_fieldset_name="correlati",
            )
        IServizio.setTaggedValue(
            OMITTED_KEY,
            [
                (Interface, "condizioni_di_servizio", "true"),
                (Interface, "canale_digitale_link", "true"),
                (Interface, "canale_fisico", "true"),
                (Interface, "timeline_tempi_scadenze", "true"),
            ],
        )
        IServizio["a_chi_si_rivolge"].required = False
        IServizio["come_si_fa"].required = False
        IServizio["cosa_si_ottiene"].required = False
        IServizio["tempi_e_scadenze"].required = False
        self.move_field(
            schema=IServizio,
            fieldname="codice_ipa",
            to_fieldset_name="categorization",
        )
        self.move_field(
            schema=IServizio,
            fieldname="settore_merceologico",
            to_fieldset_name="categorization",
        )

    def fix_unita_organizzativa_schema(self):
        """fix UnitaOrganizzativa fields"""
        IUnitaOrganizzativa["competenze"].required = False
        IUnitaOrganizzativa["sede"].required = False
        IUnitaOrganizzativa.setTaggedValue(
            OMITTED_KEY, [(Interface, "documenti_pubblici", "true")]
        )

    def fix_luogo_schema(self):
        """fix UnitaOrganizzativa fields"""
        if self.schema.getName() == "ILuogo":
            self.schema["modalita_accesso"].required = False



# -*- coding: utf-8 -*-
from collective.venue.interfaces import IVenue
from collective.volto.blocksfield.field import BlocksField
from design.plone.contenttypes.interfaces.unita_organizzativa import IUnitaOrganizzativa
from design.plone.ctgeneric import _
from plone.app.dexterity import textindexer
from plone.autoform.interfaces import IFormFieldProvider
from plone.supermodel import model
from zope import schema
from zope.component import adapter
from zope.interface import implementer
from zope.interface import provider


class IContattiV2(model.Schema):
    """ """

    telefono = schema.TextLine(
        title=_("telefono_label", default="Telefono"),
        description=_(
            "telefono_help",
            default="Indicare un riferimento telefonico per poter contattare"
            " i referenti.",
        ),
        required=False,
    )

    fax = schema.TextLine(
        title=_("fax_label", default="Fax"),
        description=_("fax_help", default="Indicare un numero di fax."),
        required=False,
    )

    email = schema.TextLine(
        title=_("email_label", default="E-mail"),
        description=_(
            "email_help",
            default="Indicare un indirizzo mail per poter contattare" " i referenti.",
        ),
        required=False,
    )

    pec = schema.TextLine(
        title=_("pec_label", default="Pec"),
        description=_(
            "pec_help",
            default="Indicare un indirizzo pec per poter contattare" " i referenti.",
        ),
        required=False,
    )

    web = schema.TextLine(
        title=_("web_label", default="Sito web"),
        description=_("web_help", default="Indicare un indirizzo web di riferimento."),
        required=False,
    )

    orario_pubblico = BlocksField(
        title=_("orario_pubblico_label", default="Orario per il pubblico"),
        description=_(
            "orario_pubblico_help",
            default="Indicare eventuali orari di accesso al pubblico",
        ),
        required=False,
    )

    textindexer.searchable("orario_pubblico")
    textindexer.searchable("email")
    textindexer.searchable("pec")
    textindexer.searchable("web")


@provider(IFormFieldProvider)
class IContattiUnitaOrganizzativaV2(IContattiV2):
    model.fieldset(
        "contatti",
        label=_("contatti_label", default="Contatti"),
        fields=["telefono", "fax", "email", "pec", "web", "orario_pubblico"],
    )


@provider(IFormFieldProvider)
class IContattiVenueV2(IContattiV2):
    model.fieldset(
        "orari",
        label=_("orari_label", default="Orari di apertura"),
        fields=["orario_pubblico"],
    )

    model.fieldset(
        "contatti",
        label=_("contatti_label", default="Contatti"),
        fields=["telefono", "fax", "email", "pec", "web"],
    )


@implementer(IContattiUnitaOrganizzativaV2)
@adapter(IUnitaOrganizzativa)
class ContattiUnitaOrganizzativaV2(object):
    """ """

    def __init__(self, context):
        self.context = context


@implementer(IContattiVenueV2)
@adapter(IVenue)
class ContattiVenueV2(object):
    """ """

    def __init__(self, context):
        self.context = context



# -*- coding: utf-8 -*-
from design.plone.ctgeneric.behaviors.persona import IPersonaV2
from plone.indexer.decorator import indexer


@indexer(IPersonaV2)
def ruolo(obj):
    """ """
    return getattr(obj.aq_base, "ruolo", "")



# -*- coding: utf-8 -*-
from design.plone.ctgeneric.interfaces import IDesignPloneV2Settings
from design.plone.ctgeneric.interfaces import IDesignPloneV2SettingsControlpanel
from plone.restapi.controlpanels import RegistryConfigletPanel
from zope.component import adapter
from zope.interface import implementer
from zope.interface import Interface


@adapter(Interface, Interface)
@implementer(IDesignPloneV2SettingsControlpanel)
class DesignPloneV2Settings(RegistryConfigletPanel):
    schema = IDesignPloneV2Settings
    configlet_id = "DesignPloneV2Settings"
    configlet_category_id = "Products"
    schema_prefix = None



# -*- coding: utf-8 -*-
from design.plone.contenttypes.tests.test_ct_document import (
    TestDocumentSchema as BaseSchemaTest,
)
from design.plone.ctgeneric.testing import DESIGN_PLONE_CTGENERIC_API_FUNCTIONAL_TESTING


class TestDocumentSchema(BaseSchemaTest):
    layer = DESIGN_PLONE_CTGENERIC_API_FUNCTIONAL_TESTING



# -*- coding: utf-8 -*-
from design.plone.ctgeneric import _
from plone.app.z3cform.widget import RelatedItemsFieldWidget
from plone.autoform import directives as form
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.interfaces import IDexterityContent
from plone.supermodel import model
from z3c.relationfield.schema import RelationChoice
from z3c.relationfield.schema import RelationList
from zope import schema
from zope.component import adapter
from zope.interface import implementer
from zope.interface import provider


@provider(IFormFieldProvider)
class IDocumentoV2(model.Schema):
    ufficio_responsabile = RelationList(
        title=_(
            "ufficio_responsabile_documento_label",
            default="Ufficio responsabile del documento",
        ),
        description=_(
            "ufficio_responsabile_documento_help",
            default="Seleziona l'ufficio responsabile di questo documento.",
        ),
        required=True,
        default=[],
        value_type=RelationChoice(
            title=_("Ufficio responsabile"),
            vocabulary="plone.app.vocabularies.Catalog",
        ),
    )
    tipologia_documento = schema.Choice(
        title=_("tipologia_documento_label", default="Tipologia del documento"),
        description=_(
            "tipologia_documento_help",
            default="Seleziona la tipologia del documento.",
        ),
        required=True,
        vocabulary="design.plone.vocabularies.tipologie_documento",
    )

    # custom widgets
    form.widget(
        "ufficio_responsabile",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "maximumSelectionSize": 1,
            "selectableTypes": ["UnitaOrganizzativa"],
        },
    )

    # custom order
    form.order_after(tipologia_documento="identificativo")
    form.order_before(ufficio_responsabile="licenza_distribuzione")

    #  custom fieldsets
    model.fieldset(
        "descrizione",
        label=_("descrizione_label", default="Descrizione"),
        fields=[
            "ufficio_responsabile",
        ],
    )


@implementer(IDocumentoV2)
@adapter(IDexterityContent)
class DocumentoV2(object):
    """ """

    def __init__(self, context):
        self.context = context



# -*- coding: utf-8 -*-
from design.plone.contenttypes.interfaces.unita_organizzativa import IUnitaOrganizzativa
from design.plone.contenttypes.restapi.serializers.unita_organizzativa import (
    UOJSONSummarySerializer as BaseSerializer,
)
from design.plone.ctgeneric.interfaces import IDesignPloneCtgenericLayer
from plone.restapi.interfaces import ISerializeToJsonSummary
from zope.component import adapter
from zope.interface import implementer


@implementer(ISerializeToJsonSummary)
@adapter(IUnitaOrganizzativa, IDesignPloneCtgenericLayer)
class UOJSONSummarySerializer(BaseSerializer):
    """
    Override dei fields
    """

    fields = [
        "contact_info",
        "sede",
        "address",
        "city",
        "zip_code",
        "email",
        "telefono",
        "nome_sede",
        "title",
        "quartiere",
        "circoscrizione",
        "street",
    ]



# -*- coding: utf-8 -*-
from collective.geolocationbehavior.geolocation import IGeolocatable
from collective.volto.blocksfield.field import BlocksField
from design.plone.contenttypes.interfaces.unita_organizzativa import IUnitaOrganizzativa
from design.plone.ctgeneric import _
from plone.autoform import directives as form
from plone.autoform.interfaces import IFormFieldProvider
from plone.supermodel import model
from zope import schema
from zope.component import adapter
from zope.interface import implementer
from zope.interface import provider


@provider(IFormFieldProvider)
class IUnitaOrganizzativaV2(model.Schema, IGeolocatable):
    contact_info = BlocksField(
        title=_("contact_info_label", default="Informazioni di contatto generiche"),
        required=False,
        description=_(
            "uo_contact_info_description",
            default="Inserisci eventuali informazioni di contatto aggiuntive "
            "non contemplate nei campi precedenti. "
            "Utilizza questo campo se ci sono dei contatti aggiuntivi rispetto"
            " ai contatti della sede principale. Se inserisci un collegamento "
            'con un indirizzo email, aggiungi "mailto:" prima dell\'indirizzo'
            ", per farlo aprire direttamente nel client di posta.",
        ),
    )
    nome_sede = schema.TextLine(
        title=_("nome_sede", default="Nome sede"),
        description=_(
            "help_nome_sede",
            default="Inserisci il nome della "
            "sede, se non è presente tra i Luoghi del sito.",
        ),
        required=False,
    )

    model.fieldset(
        "contatti",
        label=_("contatti_label", default="Contatti"),
        fields=["contact_info", "nome_sede", "geolocation"],
    )

    form.order_after(geolocation="country")


@implementer(IUnitaOrganizzativaV2)
@adapter(IUnitaOrganizzativa)
class UnitaOrganizzativaV2(object):
    """"""

    def __init__(self, context):
        self.context = context



# -*- coding: utf-8 -*-
from design.plone.contenttypes.tests.test_ct_unita_organizzativa import (
    TestUO as BaseTest,
)
from design.plone.contenttypes.tests.test_ct_unita_organizzativa import (
    TestUOSchema as BaseSchemaTest,
)
from design.plone.ctgeneric.testing import DESIGN_PLONE_CTGENERIC_API_FUNCTIONAL_TESTING
from plone import api
from transaction import commit


class TestUOSchema(BaseSchemaTest):
    layer = DESIGN_PLONE_CTGENERIC_API_FUNCTIONAL_TESTING

    def test_behaviors_enabled_for_uo(self):
        portal_types = api.portal.get_tool(name="portal_types")
        self.assertEqual(
            portal_types["UnitaOrganizzativa"].behaviors,
            (
                "plone.namefromtitle",
                "plone.allowdiscussion",
                "plone.excludefromnavigation",
                "plone.shortname",
                "plone.ownership",
                "plone.publication",
                "plone.categorization",
                "plone.basic",
                "plone.locking",
                "plone.leadimage",
                "volto.preview_image",
                "plone.relateditems",
                "design.plone.contenttypes.behavior.argomenti",
                "plone.textindexer",
                "design.plone.contenttypes.behavior.additional_help_infos",
                "plone.translatable",
                "kitconcept.seo",
                "plone.versioning",
                "design.plone.contenttypes.behavior.unita_organizzativa_v2",
                "design.plone.contenttypes.behavior.address_uo",
                "design.plone.contenttypes.behavior.geolocation_uo",
                "design.plone.contenttypes.behavior.contatti_uo_v2",
            ),
        )

    def test_uo_required_fields(self):
        resp = self.api_session.get("@types/UnitaOrganizzativa").json()
        self.assertEqual(
            sorted(resp["required"]),
            sorted(["title", "tipologia_organizzazione"]),
        )

    def test_uo_fields_struttura_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/UnitaOrganizzativa").json()
        self.assertEqual(
            resp["fieldsets"][2]["fields"],
            [
                "legami_con_altre_strutture",
                "responsabile",
                "tipologia_organizzazione",
                "assessore_riferimento",
            ],
        )

    def test_uo_fields_contatti_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/UnitaOrganizzativa").json()
        self.assertEqual(
            resp["fieldsets"][4]["fields"],
            [
                "sede",
                "sedi_secondarie",
                "contact_info",
                "nome_sede",
                "street",
                "zip_code",
                "city",
                "quartiere",
                "circoscrizione",
                "country",
                "geolocation",
                "telefono",
                "fax",
                "email",
                "pec",
                "web",
                "orario_pubblico",
            ],
        )

    def test_uo_fields_categorization_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/UnitaOrganizzativa").json()
        self.assertEqual(
            resp["fieldsets"][6]["fields"],
            # ["subjects", "language"] BBB dovrebbe essere così
            # ma nei test esce così perché non viene vista la patch di SchemaTweaks
            ["subjects", "language", "relatedItems"],
        )


class TestUO(BaseTest):
    """"""

    layer = DESIGN_PLONE_CTGENERIC_API_FUNCTIONAL_TESTING

    def test_cant_patch_uo_that_has_no_required_fields(self):
        """
        in V3 you get a 400, but here you can
        """
        uo = api.content.create(
            container=self.portal, type="UnitaOrganizzativa", title="Foo"
        )
        commit()
        resp = self.api_session.patch(
            uo.absolute_url(),
            json={
                "title": "Foo modified",
            },
        )
        self.assertEqual(resp.status_code, 204)



from .layer import IDesignPloneCtgenericLayer
from .settings import IDesignPloneV2Settings
from .settings import IDesignPloneV2SettingsControlpanel


IDesignPloneCtgenericLayer  # noqa
IDesignPloneV2SettingsControlpanel  # noqa
IDesignPloneV2Settings  # noqa
